/**
 * Create a dialog to generate a slider for choosing uninstalled apps.
 *
 * The .contextualappadsUiAppsdialog() creates a dialog which enables the user to slide between relevant apps which could be useful for his context. Hovering over a recommended app shows more information of the app.
 *
 * ### Examples
 * Binds the contextualappadsUiAppsdialog()-Dialog to the following div and calls the open-method of the widget via on-click event attached to the page element.
 *
 * HTML:
 * <div class="ep-top-bar-buttons ep-top-bar-buttons-context-apps"></div>
 *
 * JavaScript:
 *
 *    <script>
 *        require([
 *           'jquery',
 *           'ep',
 *           'de_epages/contextualappads/ui/appsdialog',
 *
 *           '$ready!'
 *       ], function($, ep, de_epages) {
 *           var appsForContext = [{
 *                   appUrl: 'https://www.billbee.de',
 *                   description: 'Billbee bietet eine umfangreiche aber einfach zu bedienende...',
 *                   id: '82F2085D-A912-4BA8-B0D8-B03713593906',
 *                   detailId: '82F2085D-A912-4BA8-B0D8-B03713593906',
 *                   name: 'Billbee',
 *                   installed: false,
 *                   logoUrl: 'bild1.jpg'
 *               }, {
 *                   appUrl: 'https://www.facebook.de',
 *                   description: 'Facebook bietet eine umfangreiche aber einfach zu bedienende...',
 *                   id: '2',
 *                   detailId: '2',
 *                   name: 'Facebook',
 *                   installed: false,
 *                   logoUrl: 'bild2.jpg'
 *               }, {
 *                   appUrl: 'https://www.whatsapp.de',
 *                   description: 'Whatsapp bietet eine umfangreiche aber einfach zu bedienende...',
 *                   id: '3',
 *                   detailId: '3',
 *                   name: 'WhatsApp',
 *                   installed: true,
 *                   logoUrl: 'bild3.jpg'
 *               }];
 *
 *           $('.ep-top-bar-buttons-context-apps').contextualappadsUiAppsdialog({
 *               apps: appsForContext,
 *               shopId: #Shop.ID
 *           });
 *       });
 *   </script>
 *
 *
 *
 * @class jQuery.ui.contextualappadsUiAppsdialog
 * @extends jQuery.widget
 *
 * @uses ep
 * @uses de_epages
 * @uses $tmpl!./appsdialog
 * @uses jquery/slick/slick
 * @uses ep.dict
 * @uses jQuery.ui.widget
 * @uses ep.ui.dialog
 * @uses ep.openWindow
 * @uses jquery/tmpl
 * @uses de_epages/contextualappads/ui/appsdialog
 * @since 7.30.0
 */

/**
 * @cfg {Object} [apps] Array of objects containing the relevant apps. When initializing the dialog, the Array will be filled with the app objects from the parsed JSON object (JSON.parse('#AppsForContext')).
 */

/**
 * @cfg {String} [viewAction] Name of the ViewAction that should be called if the read more button was clicked.
 */

/**
 * @cfg {String} [webUrlSSL] URL to validate the origin inside the iframe.
 */

/**
 * @cfg {String} [width] Maximum width of the dialog.
 */

/**
 * @cfg {Boolean} [showNumberBubble] Flag to decide if number bubble on the icon should be shown/hidden.
 */

/**
 * @cfg {String} [mboContext] Name of the context, e.g. product, order.
 */

/**
 * See `jQuery.ui.contextualappadsUiAppsdialog` for details.
 *
 * @param {Array}  [apps] Array of objects containing the relevant apps. When initializing the dialog, the Array will be filled with the app objects from the parsed JSON object (JSON.parse('#AppsForContext')).
 * @param {String} [viewAction] Name of the ViewAction that should be called if the read more button was clicked.
 * @param {String} [webUrlSSL] URL to validate the origin inside the iframe.
 * @param {String} [width] Maximum width of the dialog.
 * @param {Boolean} [showNumberBubble] Flag to decide if number bubble on the icon should be shown/hidden.
 * @param {String} [mboContext] Name of the context, e.g. product, order.
 *
 *
 *
 * ### Examples
 * apps[0] contains an app object with the following properties: (e.g. an app from Bilbee)
 *
 *      appUrl: 'https://www.billbee.de'
 *      description: 'Billbee bietet eine umfangreiche aber einfach zu bedienende...'
 *      id: 'E03F1CBF-A9E0-4D31-99DB-1D9EAFE58F64'
 *      name: 'Billbee'
 *      installed: false
 *      ...
 *
 *
 * @method contextualappadsUiAppsdialog
 * @member jQuery
 *
 * @since 7.30.0
 */

/*
 * @copyright       © Copyright 2006-2018, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages/contextualappads/ui/appsdialog
 */

define("de_epages/contextualappads/ui/appsdialog", [
    "jquery",
    "ep",
    "de_epages",
    "$tmpl!./appsdialog",
    "$dict!ep/dict",

    "jquery/slick/slick",
    "jquery/ui/widget",
    "ep/openwindow",
    "ep/ui/dialog",
    "jquery/tmpl"
], function($, ep, de_epages, tmplAppDialog, epDict) {
/*
 * @dictionary      ep.dict
 *
 * @translation     {Cancel}
 *                  {Free}
 *                  {InstallApp}
 *                  {ReadMoreButton}
 */

    // translation object
    var translations = {
            Cancel: epDict.translate('Cancel'),
            Free: epDict.translate('Free'),
            InstallApp: epDict.translate('InstallApp'),
            ReadMoreButton: epDict.translate('ReadMoreButton')
        };

    $.widget('ui.contextualappadsUiAppsdialog', {
    //  dialog,
    //  installButton,
    //  openWindow,
        tmplElements: {},
        apps: [],

        options: {
        //  mboContext: '',
            apps: [],
            viewAction: 'MBO-ViewAppDetailsFrame',
            webUrlSSL: '*',
            width: 960,
            showNumberBubble: false
        },

        /**
         * Method is called during initialization, when widget is created.
         *
         * @member jQuery.ui.contextualappadsUiAppsdialog
         *
         * @since 7.30.0
         */
        _create: function() {
            var self = this,
                o = self.options,
                elem = self.element;

            if (o.apps.length) {
                // filter apps
                // apps are copied by reference, not as a deep copy
                self.apps = o.apps.filter(function(app) {
                    // filter uninstalled apps only
                    return !app.installed;
                });

                // register click event to open the dialog
                elem.on('click.appDialog', function(evt) {
                    o.showNumberBubble && o.mboContext && self._addAppsToContext();
                    self.open();
                });
            }
        },

        /**
         * Method renders the slider template and creates the dialog.
         *
         * @member jQuery.ui.contextualappadsUiAppsdialog
         *
         * @since 7.30.0
         */
        _createDialog: function() {
            var self = this,
                o = self.options,
                elem = self.element,
                slides = (self.apps.length < 3) ? self.apps.length : 3,
                renderedTmpl;

            renderedTmpl = tmplAppDialog([{
                apps: self.apps,
                shopId: o.shopId,
                dict: translations
            }]);

            // Get DOM-Elements from template.
            $.extend(self.tmplElements, renderedTmpl.tmplItem('elements'));
            // Initialize dialog widget. (Width depending on number of apps)
            self.dialog = $('<div/>').html(renderedTmpl).uiDialog({
                width: parseInt(o.width * slides / 3),
                modal: true,
                classes: {
                  'ui-dialog': 'ep-appdialog-dialog'
                },
                position: self._getPosition(elem)
            });

            // initialize the slider plugin
            self._initSlider(slides);

            // Register live-event for ReadMore Buttons inside AppTeaser.
            self.tmplElements.sliderlist.on('click.appReadMore', '.ep-appdialog-slider-teaser-readmore', function() {
                var $this = $(this),
                    appDetails = $this.data('appdetails');

                self.close();

                // open dialog with installation iframe
                self._openInstallationDialog(self, appDetails);
            });

            // adjust position of dialog on resize
            $(window).on('resize.contextualappadsUiAppsdialog', function() {
                if (self.dialog && self.dialog.data('uiUiDialog') !== undefined) {
                    self.dialog.uiDialog('option', {
                        position: self._getPosition(elem)
                    });
                }
            });
        },

        /**
         * Method to open an individual app inside an iframe for detailed information and installation.
         *
         * @member jQuery.ui.contextualappadsUiAppsdialog
         *
         * @since 7.32.0
         */
         _openInstallationDialog: function(appDialogWidget, appDetails) {
            var o = appDialogWidget.options,
                features = {
                    iframe: {
                        height: '98%'
                    },
                    onAfterIframeLoaded: function() {
                        appDialogWidget.installButton && appDialogWidget._toggleButton(appDialogWidget.installButton, false);
                    }
                },
                additional = {
                    // default properties from openWindow plugin that needs to be overwritten
                    width: 1010,
                    height: 0.8 * $(window).height()
                    // , position: appDialogWidget._getPosition(elem)
                };

            // add buttons and some other options only on first openWindow event
            if (!appDialogWidget.openWindow) {
                // properties for the openWindow dialog that had to be set only on the first initialization
                additional = $.extend(additional, {
                    preventScroll: true,
                    maxWidth: o.width,
                    resizable: false,
                    modal:  true,
                    classes: {
                        'ui-dialog': 'ep-appdialog-detailsdialog'
                    },
                    close: function(event, ui) {
                        appDialogWidget.open();
                    },
                    open: function(event, ui) {
                        // // disable install button
                        appDialogWidget.installButton.prop('disabled', true);
                    },
                    create: function(event, ui) {
                        // get reference on install button
                        appDialogWidget.installButton = $(this).closest('.ui-dialog').find('.ep-button-app-install');
                    },
                    buttons: [
                        {
                            text: translations.Cancel,
                            cssClass: 'ep-button ep-button-close',
                            click: function(){
                                ep(this).uiDialog('close');
                            }
                        },
                        {
                            text: translations.InstallApp,
                            cssClass: 'ep-button ep-button-cta ep-button-app-install',
                            click: function(){
                                // hard coded app id (Sales App) for testing during implementation
                                // location.href = "?ViewAction=MBO-ViewAuthorizationPage&appID=82F2085D-A912-4BA8-B0D8-B03713593906&ObjectID=" + o.shopId;
                                location.href = "?ViewAction=MBO-ViewAuthorizationPage&appID=" + appDetails.appID + "&ObjectID=" + o.shopId;

                                // ToDo: uncomment the iframe communication (postMessage) if install button routine is implemented
                                //       will be done with UNITY-5322: https://epages.atlassian.net/browse/UNITY-5322

                                // var loc = o.webUrlSSL.split("?"),
                                //     hostAndPort = loc[0],
                                //     $iframe = $(this).find('iframe');

                                // // send post message to iframe
                                // $iframe[0].contentWindow.postMessage({
                                //     type: 'MBO_INSTALL_APP',
                                //     payload: true
                                // }, hostAndPort);
                            }
                        }
                    ]
                });
                appDialogWidget.openWindow = true;
            }

            // open window with iframe containing the details of the app
             ep.openWindow(
                "?ViewAction=" + o.viewAction + "&appID=" + appDetails.appID + "&ObjectID=" + o.shopId,
                // Use the following app link for testing (app-id is for Sales Analytics App)
                // ToDo: remove this link after implementation of the install button UNITY-5322
                // "?ViewAction=" + o.viewAction + "&appID=82F2085D-A912-4BA8-B0D8-B03713593906&ObjectID=" + o.shopId,
                features,
                additional
            );
         },

        /**
         * Send context and corresponding app ids via AJAX request.
         *
         * @member jQuery.ui.contextualappadsUiAppsdialog
         *
         * @since 7.33.0
         */
         _addAppsToContext: function() {
            var self = this,
                o = self.options,
                elem = self.element,
                seenAppData = {};

            // generate data object for request (example: {'product' : ['938222F2-2060-4A5D-9822-3D36D4730928', '747BE823-8CAC-4FF7-9857-7017E3DF65AE']})
            seenAppData[o.mboContext] = o.apps.map(function(app) {
                return app.id;
            });

            ep.ajax({
                type: "POST",
                dataType: "json",
                url: '?',
                data: {
                    ViewAction: "JSONViewResponse",
                    ChangeAction: "JSONSaveSeenAppIDs",
                    SeenAppIDs: JSON.stringify(seenAppData)
                }
            })
            .done(function(data, textStatus, jqXHR) {
                if (data.success) {
                    // avoid multiple server requests
                    o.showNumberBubble = false;

                    // hide number bubble
                    elem.addClass('ep-top-bar-apps-count-invisible');
                } else if (data.error) {
                    console.log('ContextualAppAds/Data/Public/lib/de_epages/contextualappads/ui/appsdialog.js: ', data.error);
                }
            })
            .fail(function(jqXHR, textStatus, errorThrown) {
                // uncomment the following line for debugging if an error occurs
                // console.log(arguments);
            });
         },

        /**
         * Method disable/enable a button.
         *
         * @member jQuery.ui.contextualappadsUiAppsdialog
         *
         * @since 7.32.0
         */
         _toggleButton: function(button, state) {
            button.prop('disabled', state);
         },

        /**
         * Method initializes the slick slider plugin.
         *
         * @member jQuery.ui.contextualappadsUiAppsdialog
         *
         * @since 7.30.0
         */
        _initSlider: function(slides) {
            var self = this;

            self.tmplElements.sliderlist.on('init', function(event, slick) {
                // show/hide arrows (depending on slick variables, see: https://github.com/kenwheeler/slick/)
                self._setArrows(slick);
            });

            // Initialize Slick Slider
            // (Config param: https://github.com/kenwheeler/slick/)
            self.tmplElements.sliderlist.slick({
              infinite: false,
              slidesToShow: slides,
              slidesToScroll: slides,
              prevArrow: '<span class="ep-slick-prev fa fa-chevron-circle-left"></span>',
              nextArrow: '<span class="ep-slick-next fa fa-chevron-circle-right"></span>',
            });

            self.tmplElements.sliderlist.on('afterChange', function(event, slick, currentSlide) {
                // show/hide arrows (depending on slick variables, see: https://github.com/kenwheeler/slick/)
                self._setArrows(slick);
            });

            // recalculate position (to avoid design/position glitches with to many apps filling the complete view port height)
            self.dialog.uiDialog('option', {
                position: self._getPosition(self.element)
            });
        },

        /**
         * Method to set slider arrow elements.
         *
         * @member jQuery.ui.contextualappadsUiAppsdialog
         *
         * @since 7.30.0
         */
         _setArrows: function(slick) {
            // if right border is reached => Hide arrow-element
            slick.$nextArrow[(slick.slideCount - slick.currentSlide <= slick.options.slidesToShow) ? 'hide' : 'show']();
            // if left border is reached => Hide arrow-element
            slick.$prevArrow[(slick.currentSlide === 0) ? 'hide' : 'show']();
        },

        /**
         * Method returns a positioning object.
         *
         * @member jQuery.ui.contextualappadsUiAppsdialog
         *
         * @since 7.30.0
         */
        _getPosition: function(ofElement) {
            return {
                my: "center top",
                at: "center bottom+20",
                of: ofElement
            };
        },

        /**
         * When the method is called it checks if there is already a dialog created or if the dialog is not defined yet. If not defined, the widget function _create() is called otherwise the dialog only opens as an overlay.
         *
         * @method open
         * @member jQuery.ui.contextualappadsUiAppsdialog
         *
         * @since 7.30.0
         */
        open: function() {
            var self = this;

            if (self.dialog) {
                self.dialog.uiDialog('open');
            } else {
                self._createDialog();
            }
        },

        /**
         * When the method is called an open dialog will be closed.
         *
         * @method close
         * @member jQuery.ui.contextualappadsUiAppsdialog
         *
         * @since 7.30.0
         */
        close: function() {
            var self = this;

            if (self.dialog) {
                self.dialog.uiDialog('close');
            }
        },

        destroy: function() {
            var self = this;

            // Deregister live-event for ReadMore Buttons inside AppTeaser.
            self.tmplElements.sliderlist.find('.ep-appdialog-slider-teaser-readmore').off('click.appReadMore')
            // Remove click-event from app button
            self.element.off('click.appDialog');

            // Destroy slider plugin.
            self.tmplElements.sliderlist.slick('unslick');
            // Destroy dialog widget.
            self.dialog.uiDialog('destroy');
            self.dialog = undefined;

            // destroy openWindow dialog
            self.openWindow && ep.destroyWindow();

            this._superApply(arguments);
        }
    });

    return de_epages;
});


